{% extends "admin/change_form.html" %}
{% load i18n admin_modify adminmedia %}

{% block extrahead %}{{ block.super }}
<script type="text/javascript" src="{{ MEDIA_URL }}admin/js/jquery.min.js"></script>
{% endblock %}

{% block bodyclass %}change-form{% endblock %}

{% block breadcrumbs %}{% endblock %}

{% block content %}
<script type="text/javascript">
django.jQuery(document).ready(function(){
	django.jQuery.ajax({
		url:'{{ ajaxurl }}',
		success: function(data) {window.location = '{{ redirecturl }}';},
		error: function(data) {$('#mc-progress-info').html("{% trans 'An error has occured. Please notify the system administrator.' %}")},
	});
});
</script>
<div id="content-main">
	<div class="module aligned campaininfo">
		<h2>{% trans "Test Sending in Progress" %}</h2>
		<p style="padding-top:10px;" id="mc-progress-info">{% blocktrans %}Please do not leave the page while the test mail is being sent.
		This process might take several minutes.{% endblocktrans %} <img src="{{ MEDIA_URL }}img/ajax-loader.gif"></p>
	</div>
</div>
{% endblock %}