{% extends "admin/change_form.html" %}
{% load i18n admin_modify adminmedia %}

{% block bodyclass %}change-form{% endblock %}

{% block breadcrumbs %}{% if not is_popup %}
<div class="breadcrumbs">
     <a href="{% url admin:index %}">{% trans "Home" %}</a> &rsaquo;
     <a href="{% url mailchimp_overview %}">{% trans "mailchimp overview" %}</a> &rsaquo;  
     {{ campaign.mc.subject|truncatewords:18 }}
</div>
{% endif %}{% endblock %}

{% block content %}
<div id="content-main">
	<style media="screen">
		.campaininfo table{
			width:100%;
		}
		.campaininfo td{
			vertical-align:top;
		}
		.campaininfo td.title{
			width:100px;
		}
	</style>
	<div class="module aligned campaininfo">
		<h2>{% trans "Campaign Information:" %} {{ campaign.mc.subject }}</h2>
		<table>
			<tr>
				<td class="title"><p><strong>{% trans "Sent Date" %}</strong></p></td>
				<td><p>{{ campaign.sent_date }}</p></td>
			</tr>
			<tr>
				<td class="title"><p><strong>{% trans "View on Mailchimp" %}</strong></p></td>
				<td><p><a target="_blank" href="http://admin.mailchimp.com/reports/summary?id={{ campaign.mc.web_id }}">{% trans "Click here" %}</p></td>
			</tr>
			{% if campaign.object %}
			<tr>
				<td class="title"><p><strong>{% trans "object" %}</strong></p></td>
				<td><p><a href="{{ campaign.get_object_admin_url }}">{{ campaign.object }}</a></p></td>
			</tr>
			{% endif %}		
		</table>
	</div>
</div>
{% endblock %}